@@ -450,12 +450,13 @@ static void torture_channel_exit_signal(void **state)
     rc = ssh_channel_request_send_signal(channel, "TERM");
     assert_ssh_return_code(session, rc);
 
-    exit_status = ssh_channel_get_exit_state(channel,
-                                             &exit_status,
-                                             &exit_signal,
-                                             &core_dumped);
+    rc = ssh_channel_get_exit_state(channel,
+                                    &exit_status,
+                                    &exit_signal,
+                                    &core_dumped);
+
     assert_ssh_return_code(session, rc);
-    assert_int_equal(exit_status, 0);
+    assert_int_equal(exit_status, (uint32_t)-1);
     assert_string_equal(exit_signal, "TERM");
     SAFE_FREE(exit_signal);
 }
@@ -574,6 +575,18 @@ static void torture_pubkey_hash(void **state)
     }
 }
 
+static void torture_openssh_banner_version(void **state)
+{
+    struct torture_state *s = *state;
+    ssh_session session = s->ssh.session;
+
+    int openssh_version = ssh_get_openssh_version(session);
+    int cmake_openssh_version = SSH_VERSION_INT(OPENSSH_VERSION_MAJOR, OPENSSH_VERSION_MINOR, 0);
+
+    assert_int_equal(openssh_version, cmake_openssh_version);
+}
+
+
 int torture_run_tests(void) {
     int rc;
     struct CMUnitTest tests[] = {
@@ -619,6 +632,9 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pubkey_hash,
                                         session_setup,
                                         session_teardown),
+        cmocka_unit_test_setup_teardown(torture_openssh_banner_version,
+                                        session_setup,
+                                        session_teardown),
     };
 
     ssh_init();
